

# EAP POINT ESTIMATES WITH STANDARD ERRORS AND RELIABILITY ESTIMATES 
# 
# X    -> vector of item scores (scored 1 to k)
# LAM  -> loading factor pattern (items x dimension factors)
# THRES -> threshold matrix (dimension thresholds x items)
# nodos -> matrix of quadrature points and weights
# index_nodos -> index matrix for nodos 
# 

# output is: 
# 
# th <- EAP trit estimate
# se <- posterior standard deviation (PSD; used as standard error)
# reli <- EAP reliability estimate 
# 
# 
source("zeros.R")
source("llgrmp.R")
source("pllgrad.R")
source("irtp.R")


eapllgrad <- function(X, LAM, THRES, nodos){

  
  n <- length(X)
  ni <- n
  nodos <-as.matrix(nodos)

  m <- nrow(LAM)
  r <- ncol(LAM)
  
  nnod <- dim(nodos)[1]
  
  
  L1 <- zeros(nnod, 1)
  
  nume_th <- zeros(1,1)
  nume_se <- zeros(1,1)
  
  TH <- zeros(1,1)
  SE <- zeros(1,1)
  RELI <- zeros(1,1)
  
  deno <- 0
  p <- zeros(ni, 1)
  
  # Primera transformacion: de FA a GRM
  
  p1 <- irtp(LAM, t(THRES))
  DISC <- p1$D
  DIF <- p1$P
  
  # Segunda trnasformacion de GRM A LL-GRM 
  
  p2 <- llgrmp(DISC, DIF)
  epsi <- p2$EPS
  inter <- p2$INTER
  
  # Convierte EPSI en THRESEPSI de Thresholds x items con fila final de 0's 
  
  THRESEPSI <- t(epsi)
  THRESEPSI <- rbind(THRESEPSI, zeros(1, ni))
  p <- c()
  for(h in 1:nnod){
    for (j in 1:ni){
      
      x <- 0
      x <- nodos[h,1]
      Q <- as.numeric(X[j])
      if ((Q-1) != 0) {
        p1 <- pllgrad(x, THRESEPSI[(Q - 1), j], DISC[j])
      }
      p2 <- 1
      if  (THRESEPSI[Q,j] != 0){
        p2 <- pllgrad(x, THRESEPSI[Q, j], DISC[j])
      }
      if ((X[j]) == min(X)){
        p[j] <- p2
      } else{
        p[j] <- p2 - p1
      }
      
    }
  
  
    L1[h,1] <- 1
    
    for(j in 1:ni){
      L1[h,1] <- L1[h, 1]*p[j]
    }
    pp <- 1
    for(i in 1:r){
      pp <- pp*nodos[h,2]
    }
    pp <- L1[h, 1]*pp
    deno <- deno + pp
    
    for(i in 1:r){
      nume_th[i] <- nume_th[i] + pp*nodos[h, 1]
      nume_se[i] <- nume_se[i] + pp*nodos[h, 1]*nodos[h,1]
    }
  }
  
  # The theta variance estimate obtained from the nodos is used 
  # to obtain the reiability etimates. With the provided nodos file is 3.4
  
  vartrue <- 3.4
  
  for(i in 1:r){
    TH[i,1] <- nume_th[i]/deno
    SE[i,1] <- sqrt(nume_se[i]/deno-(TH[i,1]*TH[i, 1]))
    RELI[i, 1] <- (vartrue - (nume_se[i]/deno -(TH[i,1])*TH[i,1]))/vartrue
    
    if(RELI[i, 1]<=0){
      RELI[i, 1] <- 0.01
    }
  }
  
  results <- list(TH=TH, SE=SE, RELI=RELI)

  return(results)
}
